=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2008-09-01 09:01:24 +0000
+++ bzrlib/tests/__init__.py	2008-09-02 04:34:23 +0000
@@ -2868,6 +2868,7 @@
                    'bzrlib.tests.test_transform',
                    'bzrlib.tests.test_transport',
                    'bzrlib.tests.test_transport_implementations',
+                   'bzrlib.tests.test_transport_log',
                    'bzrlib.tests.test_tree',
                    'bzrlib.tests.test_treebuilder',
                    'bzrlib.tests.test_tsort',

=== added file 'bzrlib/tests/test_transport_log.py'
--- bzrlib/tests/test_transport_log.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_transport_log.py	2008-09-02 04:34:23 +0000
@@ -0,0 +1,44 @@
+# Copyright (C) 2004, 2005, 2006, 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests for log+ transport decorator."""
+
+
+from bzrlib.tests import TestCaseWithMemoryTransport
+from bzrlib.trace import mutter
+from bzrlib.transport import get_transport
+
+
+class TestTransportLog(TestCaseWithMemoryTransport):
+
+    def test_log_transport(self):
+        base_transport = self.get_transport('')
+        logging_transport = get_transport('log+' + base_transport.base)
+
+        # operations such as mkdir are logged
+        mutter('where are you?')
+        logging_transport.mkdir('subdir')
+        self.assertContainsRe(self._get_log(True),
+            r'mkdir memory\+\d+://.*subdir')
+        self.assertContainsRe(self._get_log(True),
+            '  --> None')
+        # they have the expected effect
+        self.assertTrue(logging_transport.has('subdir'))
+        # and they operate on the underlying transport 
+        self.assertTrue(base_transport.has('subdir'))
+
+

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2008-08-28 15:31:34 +0000
+++ bzrlib/transport/__init__.py	2008-09-02 04:34:23 +0000
@@ -1758,6 +1758,9 @@
 register_lazy_transport('fakenfs+', 'bzrlib.transport.fakenfs',
                         'FakeNFSTransportDecorator')
 
+register_transport_proto('log+')
+register_lazy_transport('log+', 'bzrlib.transport.log', 'TransportLogDecorator')
+
 register_transport_proto('trace+')
 register_lazy_transport('trace+', 'bzrlib.transport.trace',
                         'TransportTraceDecorator')

=== modified file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	2008-08-14 02:15:31 +0000
+++ bzrlib/transport/http/__init__.py	2008-09-02 04:34:23 +0000
@@ -27,6 +27,7 @@
 import sys
 
 from bzrlib import (
+    debug,
     errors,
     ui,
     urlutils,
@@ -230,7 +231,8 @@
 
             # Turn it into a list, we will iterate it several times
             coalesced = list(coalesced)
-            mutter('http readv of %s  offsets => %s collapsed %s',
+            if 'http' in debug.debug_flags:
+                mutter('http readv of %s  offsets => %s collapsed %s',
                     relpath, len(offsets), len(coalesced))
 
             # Cache the data read, but only until it's been used

=== added file 'bzrlib/transport/log.py'
--- bzrlib/transport/log.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/log.py	2008-09-02 04:34:23 +0000
@@ -0,0 +1,149 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Transport decorator that logs transport operations to .bzr.log."""
+
+
+# see also the transportstats plugin, which gives you some summary information
+# in a machine-readable dump
+
+import StringIO
+import cStringIO
+import time
+import types
+
+from bzrlib.trace import mutter
+from bzrlib.transport.decorator import (
+    TransportDecorator,
+    )
+from bzrlib.transport.trace import (
+    DecoratorServer,
+    TransportTraceDecorator,
+    )
+
+
+
+
+class TransportLogDecorator(TransportDecorator):
+    """Decorator for Transports that logs interesting operations to .bzr.log.
+
+    In general we want to log things that usually take a network round trip
+    and may be slow.
+
+    Not all operations are logged yet.
+    """
+
+    def __init__(self, *args, **kw):
+        super(TransportLogDecorator, self).__init__(*args, **kw)
+        def _make_hook(hookname):
+            def _hook(relpath, *args, **kw):
+                return self._log_call(hookname, relpath, *args, **kw)
+            return _hook
+        for methodname in (
+            'append_bytes',
+            'append_file',
+            'copy_to',
+            'delete',
+            'get',
+            'has',
+            'iter_files_recursive',
+            'open_write_stream',
+            'mkdir',
+            'move',
+            'put_bytes', 'put_bytes_non_atomic', 'put_file put_file_non_atomic',
+            'list_dir', 'lock_read', 'lock_write',
+            'readv', 'rename', 'rmdir',
+            'stat',
+            'ulock',
+            ):
+            setattr(self, methodname, _make_hook(methodname))
+
+    @classmethod
+    def _get_url_prefix(self):
+        return 'log+'
+
+    def _log_call(self, methodname, relpath, *args, **kwargs):
+        before = time.time()
+        if kwargs:
+            kwargs_str = dict(kwargs)
+        else:
+            kwargs_str = ''
+        mutter("%s %s %s %s"
+               % (methodname, self._decorated.abspath(relpath),
+                  self._shorten(self._strip_tuple_parens(args)),
+                  kwargs_str))
+        try:
+            result = getattr(self._decorated, methodname)(relpath, *args)
+        except Exception, e:
+            mutter("  --> %s" % e)
+            mutter("      %.03fs" % (time.time() - before))
+            raise
+        return self._show_result(before, methodname, result)
+
+    def _show_result(self, before, methodname, result):
+        result_len = None
+        if isinstance(result, types.GeneratorType):
+            # eagerly pull in all the contents, so that we can measure how
+            # long it takes to get them.  this does make the behaviour a bit
+            # different, but we hope not noticably so
+            result = list(result)
+        if isinstance(result, (cStringIO.OutputType, StringIO.StringIO)):
+            val = repr(result.getvalue())
+            result_len = len(val)
+            shown_result = "%s(%s) (%d bytes)" % (result.__class__.__name__,
+                self._shorten(val), result_len)
+        elif methodname == 'readv':
+            num_hunks = len(result)
+            total_bytes = sum((len(d) for o,d in result))
+            shown_result = "readv response, %d hunks, %d total bytes" % (
+                num_hunks, total_bytes)
+            result_len = total_bytes
+        else:
+            shown_result = self._shorten(self._strip_tuple_parens(result))
+        mutter("  --> %s" % shown_result)
+        # XXX: the time may be wrong when e.g. a generator object is returned from
+        # an http readv, if the object is returned before the bulk data
+        # is read.
+        elapsed = time.time() - before
+        if result_len and elapsed > 0:
+            # this is the rate of higher-level data, not the raw network speed
+            mutter("      %9.03fs %8dkB/s" % (elapsed, result_len/elapsed/1024))
+        else:
+            mutter("      %9.03fs" % (elapsed))
+        return result
+
+    def _shorten(self, x):
+        if len(x) > 70:
+            x = x[:67] + '...'
+        return x
+
+    def _strip_tuple_parens(self, t):
+        t = repr(t)
+        if t[0] == '(' and t[-1] == ')':
+            t = t[1:-1]
+        return t
+
+
+class LogDecoratorServer(DecoratorServer):
+    """Server for testing."""
+
+    def get_decorator_class(self):
+        return TransportLogDecorator
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    return [(TransportLogDecorator, LogDecoratorServer)]

=== modified file 'bzrlib/transport/trace.py'
--- bzrlib/transport/trace.py	2008-07-04 06:05:10 +0000
+++ bzrlib/transport/trace.py	2008-09-02 04:34:23 +0000
@@ -33,10 +33,6 @@
     Not all operations are logged at this point, if you need an unlogged
     operation please add a test to the tests of this transport, for the logging
     of the operation you want logged.
-
-    Another future enhancement would be to log to bzrlib.trace.mutter when
-    trace+ is used from the command line (or perhaps as well/instead use
-    -Dtransport), to make tracing operations of the entire program easily.
     """
 
     def __init__(self, url, _decorated=None, _from_transport=None):
@@ -76,7 +72,7 @@
 
     def get(self, relpath):
         """See Transport.get()."""
-        self._activity.append(('get', relpath))
+        self._trace(('get', relpath))
         return self._decorated.get(relpath)
 
     def get_smart_client(self):
@@ -92,6 +88,7 @@
 
     def mkdir(self, relpath, mode=None):
         """See Transport.mkdir()."""
+        self._trace(('mkdir', relpath, mode))
         return self._decorated.mkdir(relpath, mode)
 
     def open_write_stream(self, relpath, mode=None):
@@ -104,7 +101,7 @@
     
     def put_bytes(self, relpath, bytes, mode=None):
         """See Transport.put_bytes()."""
-        self._activity.append(('put_bytes', relpath, len(bytes), mode))
+        self._trace(('put_bytes', relpath, len(bytes), mode))
         return self._decorated.put_bytes(relpath, bytes, mode)
 
     def listable(self):
@@ -122,7 +119,7 @@
     def readv(self, relpath, offsets, adjust_for_latency=False,
         upper_limit=None):
         """See Transport.readv."""
-        self._activity.append(('readv', relpath, offsets, adjust_for_latency,
+        self._trace(('readv', relpath, offsets, adjust_for_latency,
             upper_limit))
         return self._decorated.readv(relpath, offsets, adjust_for_latency,
             upper_limit)
@@ -151,6 +148,13 @@
         """See Transport.lock_write."""
         return self._decorated.lock_write(relpath)
 
+    def _trace(self, operation_tuple):
+        """Record that a transport operation occured.
+
+        :param operation: Tuple of transport call name and arguments.
+        """
+        self._activity.append(operation_tuple)
+
 
 class TraceServer(DecoratorServer):
     """Server for the TransportTraceDecorator for testing with."""

